package de.ozgcloud.formcycle.formdata;

import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

import de.xima.fc.form.common.XPropertyEnum;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor(access = AccessLevel.PRIVATE)
@Getter
enum ItemClass {

	HEADER("XHeader", null),
	PAGE("XPage", null),
	FOOTER("XFooter", null),
	SPAN("XSpan", XPropertyEnum.rtevalue),
	FIELD_SET("XFieldSet", XPropertyEnum.legend),
	CONTAINER("XContainer", null),
	BUTTON_LIST("XButtonList", null),
	TEXT_FIELD("XTextField", XPropertyEnum.label),
	TEXT_AREA("XTextArea", XPropertyEnum.label),
	SELECT("XSelect", XPropertyEnum.label),
	CHECKBOX("XCheckbox", XPropertyEnum.label),
	UPLOAD("XUpload", XPropertyEnum.label);

	static final Set<ItemClass> ROOT_CONTAINERS = EnumSet.of(HEADER, PAGE, FOOTER);
	static final Set<ItemClass> NESTED_CONTAINER = EnumSet.of(CONTAINER, FIELD_SET);
	static final Set<ItemClass> INPUT_CLASSES = EnumSet.of(TEXT_FIELD, TEXT_AREA, SELECT, CHECKBOX, UPLOAD);

	private static final Map<String, ItemClass> cache = new HashMap<>();

	static {
		for (ItemClass e : values()) {
			cache.put(e.className, e);
		}
	}

	private final String className;
	private final XPropertyEnum titleProperty;

	public Optional<XPropertyEnum> getTitleProperty() {
		return Optional.ofNullable(titleProperty);
	}

	public static ItemClass fromString(String className) {
		return cache.get(className);
	}

	public static boolean isRootContainer(ItemClass className) {
		return ROOT_CONTAINERS.contains(className);
	}

	public static boolean isNestedContainer(ItemClass className) {
		return NESTED_CONTAINER.contains(className);
	}

	public static boolean isTextNode(ItemClass className) {
		return className == SPAN;
	}

	public static boolean isInputNode(ItemClass className) {
		return INPUT_CLASSES.contains(className);
	}
}
