package de.ozgcloud.formcycle.errorhandling;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.IOException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.alibaba.fastjson.JSONObject;

import de.ozgcloud.formcycle.attachment.FormcycleAttachment;
import de.xima.fc.interfaces.workflow.params.INodeThrewExceptionBuilder;

class NodeThrewExceptionFactoryTest {

	@Spy
	@InjectMocks
	private NodeThrewExceptionFactory exceptionFactory;

	@Mock
	private INodeThrewExceptionBuilder exceptionBuilder;

	@Captor
	private ArgumentCaptor<String> errorKeyCaptor;
	@Captor
	private ArgumentCaptor<JSONObject> errorDataCaptor;
	@Captor
	private ArgumentCaptor<IOException> errorCauseCaptor;
	@Mock
	private FormcycleAttachment expectedAttachments;

	@BeforeEach
	void setup() {
		when(exceptionBuilder.error(errorKeyCaptor.capture(), errorDataCaptor.capture())).thenReturn(exceptionBuilder);
	}

	@Nested
	@DisplayName("When Can Not Send exception will throw")
	class TestCanNotSend {

		@BeforeEach
		void setup() {
			when(exceptionBuilder.cause(errorCauseCaptor.capture())).thenReturn(exceptionBuilder);
		}

		@Test
		void shouldAddErrorCode() {
			exceptionFactory.createCannotSendException("", mock(IOException.class));

			assertThat(errorKeyCaptor.getValue()).isEqualTo(EWorkflowElementNodeError.OZGCLOUD_CANNOT_SEND.name());
		}

		@Test
		void shouldAddExceptionMessage() {
			var message = "error";

			exceptionFactory.createCannotSendException(message, mock(IOException.class));

			assertThat(errorDataCaptor.getValue()).containsEntry(NodeThrewExceptionFactory.ERROR_MESSAGE_KEY, message);
		}

		@Test
		void shouldAddCause() {
			var causeException = mock(IOException.class);

			exceptionFactory.createCannotSendException("", causeException);

			assertThat(errorCauseCaptor.getValue()).isEqualTo(causeException);
		}
	}

	@Nested
	@DisplayName("When error by reading of attachments")
	class TestCanNotGetAttachments {

		@Test
		void shouldAddErrorCode() {
			exceptionFactory.createCannotGetAttachmentException("");

			assertThat(errorKeyCaptor.getValue()).isEqualTo(EWorkflowElementNodeError.PLUGIN_CANNOT_GET_ATTACHMENT.name());
		}

		@Test
		void shouldAddExceptionMessage() {
			var message = "error";

			exceptionFactory.createCannotGetAttachmentException(message);

			assertThat(errorDataCaptor.getValue()).containsEntry(NodeThrewExceptionFactory.ERROR_MESSAGE_KEY, message);
		}

	}
}